import time
import unittest
import colorama

COLOR_SUCCESS = colorama.Fore.GREEN
COLOR_FAIL = colorama.Fore.RED
COLOR_RESET = colorama.Fore.RESET

class ColoredTextTestResult(unittest._TextTestResult):
    
    def addSuccess(self, test):
        self.stream.write(COLOR_SUCCESS)
        unittest._TextTestResult.addSuccess(self, test)
        self.stream.write(COLOR_RESET)
    
    def addError(self, test, err):
        self.stream.write(COLOR_FAIL)
        unittest._TextTestResult.addError(self, test, err)
        self.stream.write(COLOR_RESET)
    
    def addFailure(self, test, err):
        self.stream.write(COLOR_FAIL)
        unittest._TextTestResult.addFailure(self, test, err)
        self.stream.write(COLOR_RESET)

    def printErrorList(self, flavour, errors):
            for test, err in errors:
                self.stream.writeln(self.separator1)
                self.stream.write(COLOR_FAIL)
                self.stream.writeln("%s: %s" % (flavour,self.getDescription(test)))
                self.stream.write(COLOR_RESET)
                self.stream.writeln(self.separator2)
                self.stream.writeln("%s" % err)

class ColoredTextTestRunner(unittest.TextTestRunner):
    
    def __init__(self, *args, **kwargs):
        unittest.TextTestRunner.__init__(self, *args, **kwargs)
        wrapper = colorama.AnsiToWin32(self.stream)
        if wrapper.should_wrap():
            self.stream = wrapper.stream
    
    def _makeResult(self):
        return ColoredTextTestResult(self.stream, self.descriptions, self.verbosity)
    
    def run(self, test):
        "Run the given test case or test suite."
        result = self._makeResult()
        startTime = time.time()
        test(result)
        stopTime = time.time()
        timeTaken = stopTime - startTime
        result.printErrors()
        self.stream.writeln(result.separator2)
        run = result.testsRun
        self.stream.writeln("Ran %d test%s in %.3fs" %
                            (run, run != 1 and "s" or "", timeTaken))
        self.stream.writeln()
        if not result.wasSuccessful():
            self.stream.write(COLOR_FAIL)
            self.stream.write("FAILED (")
            failed, errored = map(len, (result.failures, result.errors))
            if failed:
                self.stream.write("failures=%d" % failed)
            if errored:
                if failed: self.stream.write(", ")
                self.stream.write("errors=%d" % errored)
            self.stream.writeln(")")
        else:
            self.stream.write(COLOR_SUCCESS)
            self.stream.writeln("OK")
        self.stream.write(COLOR_RESET)
        return result

def get_test_runner(nocolor):
    if nocolor:
        return unittest.TextTestRunner
    else:
        return ColoredTextTestRunner


import sys
import os

def img_path(name):
    return os.path.join(os.path.dirname(os.path.abspath(__file__)), 'img', name)

class WatsonUIMeta(type):
    uis = []
    def __init__(self, name, bases, d):
        super(WatsonUIMeta, self).__init__(name, bases, d)
        if name != 'WatsonUI':
            self.uis.append(self)


class WatsonUI(object):
    """Base class for Watson UIs."""
    
    __metaclass__ = WatsonUIMeta
    
    name = None
    platform = None

    @classmethod
    def enabled(cls):
        return sys.platform == cls.platform
    
    def handle(self, failures, errors, total, time):
        if failures or errors:
            self.fail(failures, errors, total, time)
        else:
            self.success(total, time)
    
    def success(self, total, time):
        if total == -1:
            msg = 'Ran tests in %0.3f seconds'%time
        else:
            msg = 'Ran %s test%s in %0.3f seconds'%(total, total==1 and '' or 's', time)

        self.notify(False, 'Test Successful', msg, 'green')
    
    def fail(self, failures, errors, total, time):
        if total == -1:
            msg = 'Ran tests in %0.3f seconds'%time
        else:
            submsg = []
            if failures:
                submsg.append('failures=%s'%failures)
            if errors:
                submsg.append('errors=%s'%errors)
            msg = 'Ran %s test%s in %0.3f seconds (%s)'%(total, total==1 and '' or 's', time, ' '.join(submsg))
        self.notify(True, 'Test Failure', msg, 'red')
    
    def notify(self, failure, title, msg, icon):
        raise NotImplementedError
    
    def shutdown(self):
        pass

def load_ui(override):
    if override:
        if isinstance(override, basestring):
            override = set(override.split(','))
        for ui in WatsonUIMeta.uis:
            if ui.name in override:
                yield ui()
    else:
        for ui in WatsonUIMeta.uis:
            if ui.enabled():
                yield ui()
    


from optparse import make_option

from django.conf import settings
from django.core.management.base import BaseCommand
from django.test.utils import get_runner

try:
    from south.management.commands import patch_for_test_db_setup
except ImportError:
    patch_for_test_db_setup = lambda: None
    
from watson.core import main
from watson.runner import get_test_runner

def run_tests(*test_labels, **options):
    patch_for_test_db_setup()
    verbosity = int(options.get('verbosity', 1))
    interactive = options.get('interactive', True)
    failfast = options.get('failfast', False)
    nocolor = options.get('nocolor', False)
    TestSuiteRunner = get_runner(settings)
    
    class WatsonTestSuiteRunner(TestSuiteRunner):
        def run_suite(self, suite, **kwargs):
            return get_test_runner(nocolor)(verbosity=self.verbosity).run(suite)
        def suite_result(self, suite, result, **kwargs):
            return result
    
    test_runner = WatsonTestSuiteRunner(verbosity=verbosity, interactive=interactive, failfast=failfast)
    result = test_runner.run_tests(test_labels)
    return result

class Command(BaseCommand):
    
    option_list = BaseCommand.option_list + (
        make_option('-u', '--ui', help='Force the use of the given Watson UI'),
        make_option('--nocolor', action='store_true', default=False, help='Disable colored output.')
    )
    
    def handle(self, *test_labels, **options):
        try:
            main(options.get('ui'), run_tests, *test_labels, **options)
        except KeyboardInterrupt:
            pass



import sys
import os
import time
import traceback
from multiprocessing import Process, Queue
from threading import Thread, Lock
from Queue import Empty

from watson.ui import load_ui

_SLEEP_TIME = 1

MAGIC_RETURN_CODE = 254

class ReloaderThread(Thread):
    def __init__(self):
        super(ReloaderThread, self).__init__()
        self.daemon = False
        self._quit = False
        self._quit_lock = Lock()
        self.do_reload = False

    def run(self):
        """When this is run, it will force other
        threads to exit when any modules currently loaded change.
        """
        mtimes = {}
        while True:
            with self._quit_lock:
                if self._quit:
                    return

            for filename in filter(None, [getattr(module, '__file__', None)
                                          for module in sys.modules.values()]):
                while not os.path.isfile(filename): # Probably in an egg or zip file
                    filename = os.path.dirname(filename)
                    if not filename:
                        break
                if not filename: # Couldn't map to physical file, so just ignore
                    continue

                if filename.endswith('.pyc') or filename.endswith('.pyo'):
                    filename = filename[:-1]

                if not os.path.isfile(filename):
                    # Compiled file for non-existant source
                    continue

                mtime = os.stat(filename).st_mtime
                if filename not in mtimes:
                    mtimes[filename] = mtime
                    continue
                if mtime > mtimes[filename]:
                    print >> sys.stderr, 'Detected modification of %s, restarting.' % filename
                    self.do_reload = True
                    sys.exit(MAGIC_RETURN_CODE)
            time.sleep(_SLEEP_TIME)

    def quit(self):
        with self._quit_lock():
            self._quit = True


class RunnerThread(Thread):
    """A thread to run the provided function and push the test results
    back to a Queue.
    """

    def __init__(self, q, func, args, kwargs):
        super(RunnerThread, self).__init__()
        self.q = q
        self.func = func
        self.args = args
        self.kwargs = kwargs

    def run(self):
        try:
            start_time = time.clock()
            result = self.func(*self.args, **self.kwargs)
            end_time = time.clock()
            self.q.put({
                'failures': len(result.failures),
                'errors': len(result.errors),
                'total': result.testsRun,
                'time': end_time - start_time,
            })
        except Exception:
            traceback.print_exc()
            self.q.put({
                'failures': -1,
                'errors': -1,
                'total': -1,
                'time': 0,
            })


def reloader(q, func, args, kwargs):
    t = ReloaderThread()
    t.start()
    try:
        RunnerThread(q, func, args, kwargs).run()
    except KeyboardInterrupt:
        t.quit()
    finally:
        t.join()
        if t.do_reload:
            sys.exit(MAGIC_RETURN_CODE)


def main(ui_override, func, *args, **kwargs):
    p = None
    uis = list(load_ui(ui_override))
    try:
        while True:
            q = Queue()
            p = Process(target=reloader, args=(q, func, args, kwargs))
            p.daemon = True
            p.start()
            while True:
                try:
                    cmd = q.get(True, _SLEEP_TIME)
                    for ui in uis:
                        ui.handle(**cmd)
                except Empty:
                    # Timed out, check if we need to restart
                    if not p.is_alive():
                        if p.exitcode == MAGIC_RETURN_CODE:
                            break # This means we need to restart it
                        else:
                            return p.exitcode # Any other return code should be considered real
    finally:
        for ui in uis:
            ui.shutdown()
        if p is not None:
            p.terminate()



import subprocess

from watson.ui.base import img_path, WatsonUI

class AppleScriptError(Exception):
    pass


def run_script(script):
    proc = subprocess.Popen(['osascript', '-'], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = proc.communicate(script)
    if proc.returncode != 0:
        raise AppleScriptError('osascript failure: %s'%err)
    out = out.strip()
    if out == 'true':
        return True
    elif out == 'false':
        return False
    elif out.isdigit():
        return int(out)
    else:
        return out

def is_growl_running():
    """Check if Growl is running. Run this before any other Growl functions."""
    script = """
    tell application "System Events"
        set isRunning to (count of (every process whose bundle identifier is "com.Growl.GrowlHelperApp")) > 0
    end tell
    """
    return run_script(script)

def register_app():
    script= """
    tell application id "com.Growl.GrowlHelperApp"
     -- Make a list of all the notification types 
     -- that this script will ever send:
     set the notificationsList to {"Test Successful" , "Test Failure"}

     -- Register our script with growl.
     -- You can optionally (as here) set a default icon 
     -- for this script's notifications.
     register as application "WatsonTest" all notifications notificationsList default notifications notificationsList
    end tell"""
    run_script(script)

def notify(type, title, msg, img='green.png'):
    script= """
    on get_image(imgPath)
        set imgfd to open for access POSIX file imgPath
        set img to read imgfd as "TIFF"
        close access imgfd
        return img
    end get_image
    set noteImage to get_image("%s")
    tell application id "com.Growl.GrowlHelperApp"
     notify with name "%s" title "%s" description "%s" application name "WatsonTest" image noteImage
    end tell"""%(img_path(img), type, title, msg)
    run_script(script)

class GrowlUI(WatsonUI):
    """A Watson UI that uses Growl2. Only supported on OS X."""
    
    name = 'growl2'
    platform = 'darwin'
    
    def __init__(self):
        self.has_growl = is_growl_running()
        if self.has_growl:
            register_app()
    
    def notify(self, failure, title, msg, icon):
        self.has_growl
        if self.has_growl:
            type = failure and 'Test Failure' or 'Test Successful'
            notify(type, title, msg, icon+'.png')


import os

from setuptools import setup, find_packages

setup(
    name = 'Watson-CT',
    version = '0.1.1',
    author = 'David Michon',
    author_email = 'david.michon@gmail.com',
    packages = find_packages(),
    package_data = {'watson.ui': ['img/*']},
    url = 'http://github.com/puffin/watson',
    license='BSD',
    description='Continuous unit test runner for django',
    long_description=open('README.rst').read(),
    install_requires=[
        "Django >= 1.3",
        "colorama >= 0.2.7"
    ]
)